using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceProcess;
using System.Configuration;

namespace Bliss
{
    public static class ServiceHelper
    {
        #region StartServiceTimeout
        private static ushort defaultStartServiceTimeout = 7;
        private static ushort? _StartServiceTimeout = null;
        public static ushort StartServiceTimeout
        {
            get
            {
                if (_StartServiceTimeout == null)
                {
                    ushort parseResult;
                    if (ushort.TryParse(ConfigurationManager.AppSettings["StartServiceTimeout"], out parseResult))
                    {
                        _StartServiceTimeout = parseResult;
                    }
                }
                return _StartServiceTimeout.HasValue ? _StartServiceTimeout.Value : defaultStartServiceTimeout;
            }
            set
            {
                _StartServiceTimeout = value;
            }
        }
        #endregion
	
        public static bool IsRunning(string serviceName)
        {
            using (ServiceController svc = new ServiceController(serviceName))
            {
                return (svc.Status == ServiceControllerStatus.Running);
            }
        }
        public static bool Start(string serviceName)
        {
            TimeSpan timeout = TimeSpan.FromSeconds(StartServiceTimeout);
            return Start(serviceName, timeout);
        }
        public static bool Start(string serviceName, TimeSpan timeout)
        {
            using (ServiceController svc = new ServiceController(serviceName))
            {
                if (svc.Status != ServiceControllerStatus.Running &&
                    svc.Status != ServiceControllerStatus.StartPending)
                {
                    svc.Start();
                    try
                    {
                        svc.WaitForStatus(ServiceControllerStatus.Running, timeout);
                        return true;
                    }
                    catch (System.ServiceProcess.TimeoutException)
                    {
                        return false;
                    }
                }
                return true; // already started
            }
        }
    }
}
