using System;
using System.Text;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using Microsoft.VirtualServer.Interop;
using System.Collections.ObjectModel;
using System.Threading;
using Bliss.VirtualServer.MTA;
using Bliss.Threading;

namespace Bliss.VirtualServer.STA
{
	[Author("Anton Delsink", "anton@delsink.com")]
	public sealed class VirtualServerStaProxy : IVirtualServer
	{
		private VirtualServerMtaProxy _proxy;
		private VirtualServerMtaProxy proxy
		{
			get
			{
				return this._proxy;
			}
			set
			{
				this._proxy = value;
			}
		}

		#region Constructor
		public VirtualServerStaProxy(VirtualServerManager vsm, string virtualServerHostName)
		{
			MtaThreadUtil.EnforceSTA();

			MtaThreadUtil.TwoArgReturningDelegate<VirtualServerMtaProxy, VirtualServerManager, string> targetMethod =
				new MtaThreadUtil.TwoArgReturningDelegate<VirtualServerMtaProxy, VirtualServerManager, string>(
					VirtualServerMtaProxy.Create);

			VirtualServerMtaProxy result =
				MtaThreadUtil.InvokeOnMtaThread<VirtualServerMtaProxy, VirtualServerManager, string>(
					vsm,
					virtualServerHostName,
					targetMethod);

			this.proxy = result;
			this._VirtualServerManager = vsm;
		}
		#endregion

		#region VirtualServerManager
		private VirtualServerManager _VirtualServerManager;
		public VirtualServerManager VirtualServerManager
		{
			get
			{
				return this._VirtualServerManager;
			}
			set
			{
				this._VirtualServerManager = value;
			}
		}
		#endregion

		#region Properties
		#region VirtualMachines
		public ReadOnlyCollection<IVirtualMachine> VirtualMachines
		{
			get
			{
				MtaThreadUtil.EnforceSTA();

				List<IVirtualMachine> vmList = new List<IVirtualMachine>();
				foreach (VirtualMachineMtaProxy vm in
					MtaThreadUtil.GetMtaProperty<ReadOnlyCollection<IVirtualMachine>>(proxy, "VirtualMachines"))
				{
					vmList.Add(new VirtualMachineStaProxy(this, vm));
				}
				return new ReadOnlyCollection<IVirtualMachine>(vmList);
			}
		}
		#endregion
        #region HostName
        public string HostName
        {
            get
            {
                MtaThreadUtil.EnforceSTA();

                return MtaThreadUtil.GetMtaProperty<string>(this.proxy, "HostName");
            }
        }
        #endregion
        public string[] SearchFolders
        {
            get
            {
                MtaThreadUtil.EnforceSTA();

                return MtaThreadUtil.GetMtaProperty<string[]>(this.proxy, "SearchFolders");
            }
        }
        #endregion

        public IVirtualMachine AddExistingVMC(string fullPathAndFileName)
		{
			MtaThreadUtil.EnforceSTA();

			MtaThreadUtil.SingleArgReturningDelegate<IVirtualMachine, string> meth =
				new MtaThreadUtil.SingleArgReturningDelegate<IVirtualMachine, string>(
					proxy.AddExistingVMC);

			return
				new VirtualMachineStaProxy(
					this,
					MtaThreadUtil.InvokeOnMtaThread<IVirtualMachine, string>(
						fullPathAndFileName,
						meth));
		}
		public ITask CreateDifferencingVirtualHardDisk(string fullFileNameForNewVHD, string fullFileNameForBaseVHD)
		{
			MtaThreadUtil.EnforceSTA();

			MtaThreadUtil.TwoArgReturningDelegate<ITask, string, string> meth =
				new MtaThreadUtil.TwoArgReturningDelegate<ITask, string, string>(
					proxy.CreateDifferencingVirtualHardDisk);

			return
				new TaskStaProxy(null,
					MtaThreadUtil.InvokeOnMtaThread<ITask, string, string>(
						fullFileNameForNewVHD,
						fullFileNameForBaseVHD,
						meth));
		}
		public IVirtualMachine CreateVirtualMachine(string vmName, string vmcFolder)
		{
			MtaThreadUtil.EnforceSTA();

			MtaThreadUtil.TwoArgReturningDelegate<IVirtualMachine, string, string> methCreateVM =
				new MtaThreadUtil.TwoArgReturningDelegate<IVirtualMachine, string, string>(
					proxy.CreateVirtualMachine);

			return
				new VirtualMachineStaProxy(
					this,
					MtaThreadUtil.InvokeOnMtaThread<IVirtualMachine, string, string>(
						vmName,
						vmcFolder,
						methCreateVM));
		}

        public void AddSearchFolder(string targetPath)
        {
            MtaThreadUtil.EnforceSTA();

            MtaThreadUtil.InvokeOnMtaThread<string>(
                    targetPath,
                    new MtaThreadUtil.SingleArgActionDelegate<string>(this.proxy.AddSearchFolder));
        }
        public string GetBaseDiskFileName(string targetVhdFileName)
        {
            MtaThreadUtil.EnforceSTA();

            return
                MtaThreadUtil.InvokeOnMtaThread<string, string>(
                targetVhdFileName,
                new MtaThreadUtil.SingleArgReturningDelegate<string, string>(this.proxy.GetBaseDiskFileName));
        }
	}
}