using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;
using System.Threading;

namespace Bliss.VirtualServer
{
	/// <summary>
	/// An installation of Microsoft Virtual Server that is configurable
	/// via its management API exposed through COM (Microsoft.VirtualServer.Interop).
	/// This is to be used by a WinForms application meant to replace the 
	/// Web Application that serves as the standard user interface to VS.
	/// </summary>
	[Author("Anton Delsink", "anton@delsink.com")]
	public interface IVirtualServer
	{
		#region VirtualServerManager
		/// <summary>
		/// The Component causing this IVirtualServer instance to exist.
		/// Used mainly for passing back ITask objects for client notification.
		/// </summary>
		VirtualServerManager VirtualServerManager
		{
			get;
			set;
		}
		#endregion

		ReadOnlyCollection<IVirtualMachine> VirtualMachines { get; }

		IVirtualMachine CreateVirtualMachine(string vmName, string vmcFileName);

		ITask CreateDifferencingVirtualHardDisk(string fullFileNameForNewVHD, string fullFileNameForBaseVHD);

		IVirtualMachine AddExistingVMC(string fullPathAndFileName);

        string HostName
        {
            get;
        }
        string[] SearchFolders
        {
            get;
        }

        void AddSearchFolder(string targetPath);
        string GetBaseDiskFileName(string targetVhdFileName);
    }
}
