using System;
using System.Collections.Generic;
using System.Text;
using Bliss;
using System.Windows.Forms;
using System.Configuration;

namespace Bliss
{
    public static class VirtualServerService
    {
        private static readonly string serviceName = "Virtual Server";

        /// <summary>
        /// Queries the Service Control Manager to determine if the service
        /// is in the Running state. If not, it will attempt to Start the
        /// service if the app.config setting "StartVirtualServerServiceOnAppLaunch"
        /// has a value of "true".
        /// </summary>
        /// <param name="promptIfNot">If the app.config does not specify an auto-start of the service and the service is not in the Running state, this argument determines whether to prompt the user.</param>
        /// <returns>If the service is already started or was successfully started, true. Otherwise false.</returns>
        public static bool IsAvailable(bool promptIfNot)
        {
            try
            {
                if (!ServiceHelper.IsRunning(serviceName))
                {
                    if (!string.IsNullOrEmpty(ConfigurationManager.AppSettings["StartVirtualServerServiceOnAppLaunch"]) &&
                        ConfigurationManager.AppSettings["StartVirtualServerServiceOnAppLaunch"].ToUpper() == "TRUE")
                    {
                        return ServiceHelper.Start(serviceName);
                    }
                    else
                    {
                        if (promptIfNot)
                        {
                            DialogResult response = MessageBox.Show("The Virtual Server Service does not appear to be running. Would you like to start it now? Select Yes to attempt to start the service, No to exit.", "Virtual Server Service Status", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation);
                            if (response == DialogResult.Yes)
                            {
                                return ServiceHelper.Start(serviceName);
                            }
                        }
                    }
                    return false;
                }
                else
                {
                    return true; // .IsRunning == true
                }
            }
            catch
            {
                // TODO: Change startup to catch scenarios like "not installed", etc.
                return false; // failed to start the service; not timeout related.
            }
        }
    }
}
