using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Bliss.VirtualServer;

namespace Bliss.SharedUI
{
    public partial class GroupViewUserControl : UserControl
    {
        private List<IVirtualMachine> virtualMachines = new List<IVirtualMachine>();
        #region Constructor
        public GroupViewUserControl()
        {
            InitializeComponent();
        }
        #endregion

        #region Form Load
        private void GroupViewUserControl_Load(object sender, EventArgs e)
        {
            InitializeGroupsTree();
        }
        #region InitializeGroupsTree
        private void InitializeGroupsTree()
        {
            this.SuspendLayout();
            
            // load from app.config
            string configuration = System.Configuration.ConfigurationManager.AppSettings["GroupStrings"];
            if (!string.IsNullOrEmpty(configuration))
            {
                string[] cfgGroups = configuration.Split('|');
                _Groups = new List<string>(cfgGroups);
            }

            tvGroups.Nodes.Clear();
            foreach (string group in this.Groups)
            {
                tvGroups.Nodes.Add(group);
            }

            foreach (IVirtualMachine vm in this.virtualMachines)
            {
                this.Add(vm);
            }
            this.ResumeLayout();
        }
        #endregion
        #endregion

        #region Groups
        private List<string> _Groups = null;
        [Browsable(false)]
        public string[] Groups
        {
            get
            {
                if (_Groups == null) // on first access:
                {
                    _Groups = new List<string>();
                }

                return _Groups.ToArray();
            }
            set
            {
                _Groups = new List<string>();
                foreach (string group in value)
                {
                    if (!string.IsNullOrEmpty(group))
                    {
                        _Groups.Add(group);
                    }
                }
                InitializeGroupsTree();
            }
        }
        #endregion

        #region Add VirtualMachine
        public void Add(IVirtualMachine vm)
        {
            if (vm == null) return;

            if (!this.virtualMachines.Contains(vm))
            {
                this.virtualMachines.Add(vm);
            }

            string groupType = System.Configuration.ConfigurationManager.AppSettings["GroupType"];
            if (string.IsNullOrEmpty(groupType) ||
                (groupType != "StartsWith" && groupType != "EndsWith" && groupType != "SubString"))
            {
                groupType = "StartsWith";
            }

            foreach (TreeNode n in tvGroups.Nodes)
            {
                switch (groupType)
                {
                    case "StartsWith":
                        if (vm.Name.ToUpper().StartsWith(n.Text.ToUpper()))
                        {
                            n.Nodes.Add(new VirtualMachineTreeNode(vm));
                            return;
                        }
                        break;
                    case "EndsWith":
                        if (vm.Name.ToUpper().EndsWith(n.Text.ToUpper()))
                        {
                            n.Nodes.Add(new VirtualMachineTreeNode(vm));
                            return;
                        }
                        break;
                    case "SubString":
                        if (vm.Name.ToUpper().Contains(n.Text.ToUpper()))
                        {
                            n.Nodes.Add(new VirtualMachineTreeNode(vm));
                            return;
                        }
                        break;
                }
            }
            // if VM does not fit any known classification, add under root:
            tvGroups.Nodes.Add(new VirtualMachineTreeNode(vm));
        }
        #endregion

        #region AfterSelect Event
        public event EventHandler<VirtualMachineEventArgs> AfterSelect;
        private void tvGroups_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (AfterSelect != null)
            {
                if (e.Node is VirtualMachineTreeNode)
                {
                    VirtualMachineTreeNode tnVM = e.Node as VirtualMachineTreeNode;
                    AfterSelect(this, new VirtualMachineEventArgs(tnVM.VM));
                }
                else
                {
                    AfterSelect(this, new VirtualMachineEventArgs(null));
                }
            }
        }
        public class VirtualMachineEventArgs : EventArgs
        {
            public readonly IVirtualMachine VirtualMachine;
            internal VirtualMachineEventArgs(IVirtualMachine vm)
            {
                this.VirtualMachine = vm;
            }
        }
        #endregion

        private class VirtualMachineTreeNode : TreeNode
        {
            public readonly IVirtualMachine VM;
            public VirtualMachineTreeNode(IVirtualMachine vm)
            {
                this.VM = vm;
                this.Text = vm.Name;
            }
        }

        private void tvGroups_KeyPress(object sender, KeyPressEventArgs e)
        {
        }

        private void tvGroups_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.F2)
            {
                tvGroups.SelectedNode.BeginEdit();
            }
        }
    }
}