using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Configuration;
using Bliss.VirtualServer;

namespace Bliss.SharedUI
{
    public partial class VirtualHardDiskSelectorUserControl : UserControl
    {
        #region Constructor
        public VirtualHardDiskSelectorUserControl()
        {
            InitializeComponent();
        }
        #endregion

        #region Load Event
        private void VirtualHardDiskSelectorUserControl_Load(object sender, EventArgs e)
        {
            if (this.TheVirtualServerManager != null)
            {
                foreach (IVirtualMachine vm in TheVirtualServerManager.VirtualServer.VirtualMachines)
                {
                    ucGroupedVirtualMachines.Add(vm);
                }
                foreach (string path in TheVirtualServerManager.VirtualServer.SearchFolders)
                {
                    cmbSearchFolders.Items.Add(path);
                }
            }
        }
        #endregion

        #region TheVirtualServerManager
        private VirtualServerManager _TheVirtualServerManager = null;
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public VirtualServerManager TheVirtualServerManager
        {
            get { return _TheVirtualServerManager; }
            set { _TheVirtualServerManager = value; }
        }
        #endregion

        #region Browse for VHD
        private void btnBrowse_Click(object sender, EventArgs e)
        {
            if (dlgOpenFile.ShowDialog() == DialogResult.OK)
            {
                txtBrowseTarget.Text = dlgOpenFile.FileName;
            }
        }
        private void txtBrowseTarget_TextChanged(object sender, EventArgs e)
        {
            if (File.Exists(txtBrowseTarget.Text) &&
                txtBrowseTarget.Text.EndsWith(".vhd"))
            {
                OnFileSelected(txtBrowseTarget.Text);
            }
            else
            {
                OnFileSelected(string.Empty);
            }
        }
        #endregion

        #region Search Folders
        private void cmbSearchFolders_SelectedIndexChanged(object sender, EventArgs e)
        {
            RefreshList();
        }
        private void RefreshList()
        {
            if (cmbSearchFolders.SelectedItem == null) return;

            lstVHDs.Items.Clear();

            string targetPath = cmbSearchFolders.SelectedItem.ToString();

            lstVHDs.BackColor = (Directory.Exists(targetPath)) ? Color.White : Color.Yellow;

            DirectoryInfo diTarget = new DirectoryInfo(targetPath);
            if (chkSubFolders.Checked)
            {
                AddToList(diTarget, 1);
            }
            else
            {
                AddToList(diTarget, 1);
            }
        }
        private void chkSubFolders_CheckedChanged(object sender, EventArgs e)
        {
            if (!chkSubFolders.Checked)
            {
                nudMaxFolderDepth.Value = 1;
            }
            RefreshList();
        }
        private void AddToList(DirectoryInfo target, uint depth)
        {
            if (!target.Exists) return;

            try
            {
                foreach (FileInfo vhd in target.GetFiles("*.vhd"))
                {
                    lstVHDs.Items.Add(vhd.FullName);
                }
                if (depth < nudMaxFolderDepth.Value)
                {
                    foreach (DirectoryInfo di in target.GetDirectories())
                    {
                        AddToList(di, depth + 1);
                    }
                }
            }
            catch (UnauthorizedAccessException)
            {
                // we don't care, we're searching
            }
        }
        private void nudMaxFolderDepth_ValueChanged(object sender, EventArgs e)
        {
            chkSubFolders.Checked = (nudMaxFolderDepth.Value != 1);
            RefreshList();
        }
        private void btnAddFolder_Click(object sender, EventArgs e)
        {
            if (dlgSelectFolder.ShowDialog() == DialogResult.OK)
            {
                cmbSearchFolders.SelectedIndex =
                    cmbSearchFolders.Items.Add(
                        dlgSelectFolder.SelectedPath);

                if (chkAddFolderPermanently.Checked)
                {
                    TheVirtualServerManager.VirtualServer.AddSearchFolder(dlgSelectFolder.SelectedPath);
                }
            }
        }
        #endregion

        #region VHDs attached to VMs
        private void ucGroupedVirtualMachines_AfterSelect(object sender, Bliss.SharedUI.GroupViewUserControl.VirtualMachineEventArgs e)
        {
            lstVirtualMachineConnectedDisks.Items.Clear();
            if (e.VirtualMachine != null)
            {
                foreach (IHardDiskConnection vhdConn in e.VirtualMachine.HardDiskConnections)
                {
                    lstVirtualMachineConnectedDisks.Items.Add(vhdConn.VirtualHardDisk.File);
                }
                if (lstVirtualMachineConnectedDisks.Items.Count > 0)
                {
                    lstVirtualMachineConnectedDisks.SelectedIndex = 0;
                }
            }
        }
        private void lstVirtualMachineConnectedDisks_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstVirtualMachineConnectedDisks.SelectedItem != null)
            {
                string sourceFileName = lstVirtualMachineConnectedDisks.SelectedItem.ToString();

                if (!File.Exists(sourceFileName))
                {
                    lstVirtualMachineConnectedDisks.BackColor = Color.Yellow;
                }
                else
                {
                    lstVirtualMachineConnectedDisks.BackColor = Color.White;
                }

                if (File.Exists(sourceFileName) &&
                    sourceFileName.EndsWith(".vhd"))
                {
                    OnFileSelected(sourceFileName);
                }
                else
                {
                    OnFileSelected(string.Empty);
                }
            }
        }
        private void lstVHDs_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstVHDs.SelectedItem != null)
            {
                string sourceFileName = lstVHDs.SelectedItem.ToString();
                if (File.Exists(sourceFileName) &&
                    sourceFileName.EndsWith(".vhd"))
                {
                    OnFileSelected(sourceFileName);
                }
                else
                {
                    OnFileSelected(string.Empty);
                }
            }
        }
        #endregion

        //switch (tcDiffOptions.SelectedTab.Tag.ToString())
        //{
        //    case "Browse":
        //        baseFileName = txtBrowseTarget.Text;
        //        break;
        //    case "SearchFolders":
        //        baseFileName = lstVHDs.SelectedItem.ToString();
        //        break;
        //    case "VirtualMachineConnectedDisks":
        //        baseFileName = lstVirtualMachineConnectedDisks.SelectedItem.ToString();
        //        break;
        //}

        #region SelectedFileName
        private string _SelectedFileName = string.Empty;
        public string SelectedFileName
        {
            get
            {
                return this._SelectedFileName;
            }
        }
        #endregion
	
        #region FileSelected Event
        protected void OnFileSelected(string fileName)
        {
            this._SelectedFileName = fileName;
            if (FileSelected != null)
            {
                FileSelected(this, new FileSelectedEventArg(fileName));
            }
        }
        public event EventHandler<FileSelectedEventArg> FileSelected;
        public class FileSelectedEventArg : EventArgs
        {
            public readonly string FileName;
            internal FileSelectedEventArg(string fileName)
            {
                this.FileName = fileName;
            }
        }
        #endregion

    }
}
