using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VirtualServer.Interop;

namespace DTL.VirtualServer
{
	public sealed class Task
	{
		private readonly VMTask task;
		
		#region VM
		private readonly VirtualMachine _VM;
		public VirtualMachine VM
		{
			get { return _VM; }
		}
		#endregion

		#region VS
		private readonly VirtualServerHost _VS;
		public VirtualServerHost VS
		{
			get { return _VS; }
		}
		#endregion

		#region Constructor
		internal Task(VirtualMachine vm, VMTask task)
			: this(task)
		{
			this._VM = vm;
			this._VS = null; // to be null means the task is a VM-specific task.
			SaveTask();
		}
		internal Task(VirtualServerHost vs, VMTask task)
			: this(task)
		{
			this._VM = null; // to be null means the task is a Server-specific task.
			this._VS = vs;
			SaveTask();
		}
		private Task(VMTask task)
		{
			if (task == null) throw new ArgumentNullException("task");
			this.task = task;
		}
		private void SaveTask()
		{
			VirtualServerHost host = this._VS;
			if (host == null) host = this._VM.VS;

			host.RememberTask(this);
		}
		#endregion

		#region Cancel
		public bool IsCancelable
		{
			get
			{
				return this.task.IsCancelable;
			}
		}
		public void Cancel()
		{
			if (!this.task.IsCancelable)
			{
				throw new NotSupportedException("This task does not support the Cancel operation.");
			}
			else
			{
				this.task.Cancel();
			}
		}
		#endregion

		#region WaitForCompletion
		public void WaitForCompletion(int timeout)
		{
			this.task.WaitForCompletion(timeout);
		}
		#endregion

		#region Result
		[CLSCompliant(false)]
		public VMTaskResult Result
		{
			get
			{
				return this.task.Result;
			}
		}
		#endregion

		#region Progress
		private int lastProgress = 0;
		internal bool progressChanged = false; 
		public int Progress
		{
			get
			{
				// TODO: Vfy that this will never be negative
				int latestProgress = this.task.PercentCompleted;
				
				progressChanged = (latestProgress != lastProgress);

				lastProgress = latestProgress;
				return latestProgress;
			}
		}
		#endregion

		#region Description
		public string Description
		{
			get
			{
				return this.task.Description;
			}
		}
		#endregion

		#region IsComplete
		public bool IsComplete
		{
			get
			{
				return this.task.IsComplete;
			}
		}
		#endregion
	}
}