using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using DTL.VirtualServer;
using System.IO;

namespace VirtualServerUI
{
	public partial class VirtualMachineConsoleForm : Form
	{
		private VirtualMachineUserControl SelectedVirtualMachineUserControl = null;

		#region Constructor
		public VirtualMachineConsoleForm()
		{
			InitializeComponent();

			//System.Threading.Thread.CurrentThread.SetApartmentState(System.Threading.ApartmentState.STA);
		}
		#endregion
		
		#region VS
		private VirtualServerHost _VS;
		public VirtualServerHost VS
		{
			get
			{
				return _VS;
			}
			set
			{
				_VS = value;
			}
		}
		#endregion

		private void VirtualMachineConsoleForm_Load(object sender, EventArgs e)
		{
			InitGroupsTree();
			RemoveGroupsTree(); // yes, this is for visibility. init, then hide.

			foreach (VirtualMachine vm in this.VS.VirtualMachines)
			{
				AddVirtualMachineToFlowLayoutPanelOnLeft(vm);
				AddVirtualMachineToTreeViewOnRight(vm);				
			}
		}
		
		#region Flow Layout Panel Helpers
		
		private void AddVirtualMachineToFlowLayoutPanelOnLeft(VirtualMachine vm)
		{
			VirtualMachineUserControl ucVM = new VirtualMachineUserControl();
			ucVM.VM = vm;
			ucVM.Selected += new VirtualMachineUserControl.VirtualMachineUserControlSelectedDelegate(ucVM_Selected);
			flpVMs.Controls.Add(ucVM);
		}

		void ucVM_Selected(object sender, EventArgs e)
		{
			if (SelectedVirtualMachineUserControl != null)
			{
				SelectedVirtualMachineUserControl.IsSelected = false;
			}

			VirtualMachineUserControl ucVM = sender as VirtualMachineUserControl;
			SelectedVirtualMachineUserControl = ucVM;

			if (SelectedVirtualMachineUserControl.VM.State == VirtualMachineState.Paused ||
				SelectedVirtualMachineUserControl.VM.State == VirtualMachineState.Saved ||
				SelectedVirtualMachineUserControl.VM.State == VirtualMachineState.TurnedOff)
			{
				btnStart.Enabled = true;
			}

			SyncVirtualMachineSelection();
		}

		private void SyncVirtualMachineSelection()
		{
			VirtualMachineTreeNode tnVM = tvGroups.SelectedNode as VirtualMachineTreeNode;
			if (tnVM != null &&
				tnVM.VM == SelectedVirtualMachineUserControl.VM)
			{
				// already in sync
				return;
			}

			foreach (TreeNode topLevelNode in tvGroups.Nodes)
			{
				if (topLevelNode is VirtualMachineTreeNode)
				{
					if (topLevelNode.Text == SelectedVirtualMachineUserControl.VM.Name)
					{
						tvGroups.SelectedNode = topLevelNode;
						topLevelNode.EnsureVisible();
						topLevelNode.TreeView.Select();
						return;
					}
				}
				else // it is a TreeNode and likely to have children
				{
					foreach (VirtualMachineTreeNode vmNode in topLevelNode.Nodes)
					{
						if (vmNode.Text == SelectedVirtualMachineUserControl.VM.Name)
						{
							tvGroups.SelectedNode = vmNode;
							vmNode.EnsureVisible();
							vmNode.TreeView.Select();
							return;
						}
					}
				}
			}
		}
		#endregion

		private void btnStart_Click(object sender, EventArgs e)
		{
			if (SelectedVirtualMachineUserControl != null)
			{
				SelectedVirtualMachineUserControl.VM.Start();

				Program.LaunchVMRC("localhost", 5900,
					SelectedVirtualMachineUserControl.VM.Name);
			}
		}

		private void btnNew_Click(object sender, EventArgs e)
		{

		}

		private void btnSettings_Click(object sender, EventArgs e)
		{

		}

		private void btnRemove_Click(object sender, EventArgs e)
		{

		}

		private void btnGroups_Click(object sender, EventArgs e)
		{
			if (this.Controls.Contains(tvGroups))
			{
				RemoveGroupsTree();
				btnGroups.Text = "&Groups >>";
			}
			else // add control and reverse Details button arrows
			{
				AddGroupsTree();
				btnGroups.Text = "&Groups <<";
			}
		}
		
		#region Groups TreeView Helpers
		
		private int initialFormWidth = 355;
		private int expandedFormWidth = 620;
		private int initialGroupsTreeWidth = 260;

		private void AddGroupsTree()
		{
			this.Width = expandedFormWidth;
			this.Controls.Add(tvGroups);
			tvGroups.Width = initialGroupsTreeWidth;
			tvGroups.Height = flpVMs.Height;
			tvGroups.Visible = true;
			tvGroups.ResumeLayout();
		}

		private void RemoveGroupsTree()
		{
			tvGroups.Visible = false;
			tvGroups.SuspendLayout();
			this.Controls.Remove(tvGroups);

			this.Width = initialFormWidth;
		}

		string[] groupList = System.Configuration.ConfigurationManager.AppSettings["GroupStrings"].Split('|');

		private void InitGroupsTree()
		{
			foreach (string group in groupList)
			{
				if (!string.IsNullOrEmpty(group))
				{
					tvGroups.Nodes.Add(group);
				}
			}
		}

		private void AddVirtualMachineToTreeViewOnRight(VirtualMachine vm)
		{
			foreach (TreeNode n in tvGroups.Nodes)
			{
				switch (System.Configuration.ConfigurationManager.AppSettings["GroupType"])
				{
					case "StartsWith":
						if (vm.Name.ToUpper().StartsWith(n.Text.ToUpper()))
						{
							n.Nodes.Add(new VirtualMachineTreeNode(vm));
							return;
						}
						break;
					case "EndsWith":
						if (vm.Name.ToUpper().EndsWith(n.Text.ToUpper()))
						{
							n.Nodes.Add(new VirtualMachineTreeNode(vm));
							return;
						}
						break;
					case "SubString":
						if (vm.Name.ToUpper().Contains(n.Text.ToUpper()))
						{
							n.Nodes.Add(new VirtualMachineTreeNode(vm));
							return;
						}
						break;
				}
			}
			tvGroups.Nodes.Add(new VirtualMachineTreeNode(vm));
		}

		private class VirtualMachineTreeNode : TreeNode
		{
			public readonly VirtualMachine VM;
			public VirtualMachineTreeNode(VirtualMachine vm)
			{
				this.VM = vm;
				this.Text = vm.Name;
			}
		}

		#endregion

		private void tvGroups_AfterSelect(object sender, TreeViewEventArgs e)
		{
			VirtualMachineTreeNode vmNode = e.Node as VirtualMachineTreeNode;
			
			mnuCreateDiffVM.Enabled = (vmNode != null);

			if (SelectedVirtualMachineUserControl != null &&
				vmNode.VM == SelectedVirtualMachineUserControl.VM)
			{
				// already in sync
				return;
			}

			foreach (VirtualMachineUserControl ucVM in flpVMs.Controls)
			{
				if (ucVM.VM.Name == e.Node.Text)
				{
					ucVM.OnSelect();
					flpVMs.ScrollControlIntoView(ucVM);
					break;
				}
			}
		}

		private void mnuCreateDiffVM_Click(object sender, EventArgs e)
		{
			FolderBrowserDialog dlgFolder = new FolderBrowserDialog();
			dlgFolder.Description = "Select folder for Difference drive.";
			dlgFolder.ShowNewFolderButton = true;
			DialogResult response = dlgFolder.ShowDialog();
			if (response == DialogResult.OK)
			{
				string diffFolder = dlgFolder.SelectedPath;

				VirtualMachineTreeNode tn = tvGroups.SelectedNode as VirtualMachineTreeNode;
				VirtualMachine targetVM = tn.VM;

				dlgFolder.Description = "Select folder for VMC/VSV files.";
				response = dlgFolder.ShowDialog();
				if (response == DialogResult.OK)
				{
					string vmcFolder = dlgFolder.SelectedPath;

					VirtualMachine newVM = targetVM.VS.CreateVirtualMachine(targetVM.Name + " HighPerf " + Guid.NewGuid().ToString(), vmcFolder);
					newVM.Memory = targetVM.Memory;
					newVM.Undoable = false;
					foreach (HardDiskConnection vhdConn in targetVM.HardDiskConnections)
					{
						FileInfo f = new FileInfo(vhdConn.VirtualHardDisk.File);
						string diffFileName = diffFolder + "\\" + f.Name.Replace(".vhd", ".diff.vhd");
						Task t = targetVM.VS.CreateDifferencingVirtualHardDisk(diffFileName, f.FullName);
						t.WaitForCompletion(-1);
						newVM.AddHardDiskConnection(diffFileName, vhdConn.BusType, vhdConn.BusNumber, vhdConn.DeviceNumber);
					}
				}
			}
		}
	}
}