using System;
using System.Text;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using Microsoft.VirtualServer.Interop;
using System.Collections.ObjectModel;
using System.Threading;
using Delsink.Threading;

namespace Delsink.VirtualServer.MTA
{
	[Author("Anton Delsink", "anton@delsink.com")]
	public sealed class VirtualServerMtaProxy : IVirtualServer
	{
		private readonly IVMVirtualServer VS; // target VS installation; the host being managed; COM interop here!

		#region Constructor
		internal VirtualServerMtaProxy(VirtualServerManager vsm, string virtualServerHostName)
		{
			MtaThreadUtil.EnforceMTA();

			this._VirtualServerManager = vsm;
			this.VS = InitVS.GetVMVirtualServerClass(virtualServerHostName);
		}
		internal static VirtualServerMtaProxy Create(VirtualServerManager vsm, string virtualServerHostName)
		{
			MtaThreadUtil.EnforceMTA();

			return new VirtualServerMtaProxy(vsm, virtualServerHostName);
		}
		#endregion

		#region Properties
		#region VirtualMachines
		public ReadOnlyCollection<IVirtualMachine> VirtualMachines
		{
			get
			{
				MtaThreadUtil.EnforceMTA();

				return new ReadOnlyCollection<IVirtualMachine>(
					GetVirtualMachines(this));
			}
		}
		#region GetVirtualMachines
		private static List<IVirtualMachine> GetVirtualMachines(VirtualServerMtaProxy targetVSH)
		{
			MtaThreadUtil.EnforceMTA();

			List<IVirtualMachine> vmList = new List<IVirtualMachine>();
			foreach (IVMVirtualMachine vm in targetVSH.VS.VirtualMachines)
			{
				vmList.Add(new VirtualMachineMtaProxy(targetVSH, vm));
			}
			return vmList;
		}
		#endregion
		#endregion
		#region VirtualServerManager
		private VirtualServerManager _VirtualServerManager;
		public VirtualServerManager VirtualServerManager
		{
			get
			{
				return this._VirtualServerManager;
			}
			set
			{
				this._VirtualServerManager = value;
			}
		}
		#endregion
		#endregion

		public IVirtualMachine AddExistingVMC(string fullFileName)
		{
			MtaThreadUtil.EnforceMTA();

			return new VirtualMachineMtaProxy(this, this.VS.RegisterVirtualMachine(null, fullFileName));
		}
		public IVirtualMachine CreateVirtualMachine(string vmName, string vmcFolder)
		{
			MtaThreadUtil.EnforceMTA();

			return new VirtualMachineMtaProxy(this, this.VS.CreateVirtualMachine(vmName, vmcFolder));
		}
		public ITask CreateDifferencingVirtualHardDisk(string fullFileNameForNewVHD, string fullFileNameForBaseVHD)
		{
			MtaThreadUtil.EnforceMTA();

			TaskMtaProxy t = new TaskMtaProxy(this,
				this.VS.CreateDifferencingVirtualHardDisk(
					fullFileNameForNewVHD,
					fullFileNameForBaseVHD));

			return t;
		}
	}
}