using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VirtualServer.Interop;
using System.Threading;
using Delsink.VirtualServer.MTA;
using Delsink.Threading;

namespace Delsink.VirtualServer.STA
{
	[Author("Anton Delsink", "anton@delsink.com")]
	public sealed class TaskStaProxy : ITask
	{
		private TaskMtaProxy proxy;

		#region VM
		private VirtualMachineStaProxy _VM;
		public IVirtualMachine VM
		{
			get
			{
				MtaThreadUtil.EnforceSTA();
				return this._VM;
			}
		}
		#endregion

		#region VS
		private VirtualServerStaProxy _VS;
		public IVirtualServer VS
		{
			get
			{
				MtaThreadUtil.EnforceSTA();
				return this._VS;
			}
		}
		#endregion

		#region Constructor
		public TaskStaProxy(IVirtualMachine vm, ITask proxy)
		{
			if (!(vm is VirtualMachineStaProxy)) throw new ArgumentException();
			if (!(proxy is TaskMtaProxy)) throw new ArgumentException();

			this._VM = (VirtualMachineStaProxy)vm;
			this._VS = (VirtualServerStaProxy)vm.VS;
			this.proxy = (TaskMtaProxy)proxy;

			SaveTask();
		}
		private void SaveTask()
		{
			VS.VirtualServerManager.RememberTask(this);
		}
		#endregion

		#region Cancel
		public bool IsCancelable
		{
			get
			{
				MtaThreadUtil.EnforceSTA();

				return MtaThreadUtil.GetMtaProperty<bool>(this.proxy, "IsCancelable");
			}
		}
		public void Cancel()
		{
			MtaThreadUtil.EnforceSTA();

			MtaThreadUtil.InvokeOnMtaThread(new ThreadStart(this.proxy.Cancel));
		}
		#endregion

		#region WaitForCompletion
		public void WaitForCompletion(int timeout)
		{
			MtaThreadUtil.EnforceSTA();

			// int return type is not in use; .WaitForCompletion returns void at all levels
			MtaThreadUtil.InvokeOnMtaThread<int>(
			    timeout,
			    new MtaThreadUtil.SingleArgActionDelegate<int>(this.proxy.WaitForCompletion));
		}
		#endregion

		#region Result
		public ITaskResult Result
		{
			get
			{
				throw new NotImplementedException();

				//return new TaskResult(this.task.Result);
			}
		}
		#endregion

		#region Progress
		public bool ProgressChanged
		{
			get
			{
				MtaThreadUtil.EnforceSTA();

				return MtaThreadUtil.GetMtaProperty<bool>(this.proxy, "ProgressChanged");
			}
		}
		public int Progress
		{
			get
			{
				MtaThreadUtil.EnforceSTA();

				return MtaThreadUtil.GetMtaProperty<int>(this.proxy, "Progress");
			}
		}
		#endregion

		#region Description
		public string Description
		{
			get
			{
				MtaThreadUtil.EnforceSTA();

				return MtaThreadUtil.GetMtaProperty<string>(this.proxy, "Description");
			}
		}
		#endregion

		#region IsComplete
		public bool IsComplete
		{
			get
			{
				MtaThreadUtil.EnforceSTA();

				return MtaThreadUtil.GetMtaProperty<bool>(this.proxy, "IsComplete");
			}
		}
		#endregion

		public override string ToString()
		{
			MtaThreadUtil.EnforceSTA();

			MtaThreadUtil.NoArgReturningDelegate<string> meth =
				new MtaThreadUtil.NoArgReturningDelegate<string>(this.proxy.ToString);

			return MtaThreadUtil.InvokeOnMtaThread<string>(meth);
		}
	}
}