using System;
using System.Collections.Generic;
using System.Drawing;

namespace Delsink.VirtualServer
{
	/// <summary>
	/// Represents the configuration of a VirtualMachine
	/// </summary>
	[Author("Anton Delsink", "anton@delsink.com")]
	public interface IVirtualMachine
	{
		string Name { get; }
		int Memory { get; set; }
		Bitmap Thumbnail { get; }
		VirtualMachineState State { get; }
		string ConfigurationFilePath { get; }

		List<IHardDiskConnection> HardDiskConnections { get; }
		List<IDvdDrive> DvdDrives { get; }
		void AddHardDiskConnection(string fullFileName, DriveBusType bt, int busNumber, int deviceNumber);

		bool Undoable { get; set; }

		#region State Change Methods
		ITask MergeUndoDisks();
		void Pause();
		ITask Reset();
		void Resume();
		ITask SaveState();
		ITask Start();
		ITask TurnOff();
		void DiscardSavedState();
		void DiscardUndoDisks();
		#endregion

        IVirtualMachine Clone();
        IVirtualMachine Clone(string diffHardDiskOutputFolder);

		IVirtualServer VS { get; }
	}
}
