using System;
using System.Collections.Generic;
using System.Windows.Forms;

using Delsink.VirtualServer.MTA;
using System.Threading;
using System.Diagnostics;
using System.Configuration;
using Delsink;

namespace Bliss.VPC
{
	[Author("Anton Delsink", "anton@delsink.com")]
	static class Program
	{
		/// <summary>
		/// Immediately initialize security for COM Interop and then start 
		/// a new thread dedicated to the UI with ApartmentState set to STA.
        /// For this to work in Debug mode, Visual Studio 2005's hosting
        /// process needs to be disabled.
		/// </summary>
        [MTAThread]
        static void Main()
        {
            Delsink.VirtualServerManager.Initialize();

            if (!string.IsNullOrEmpty(ConfigurationManager.AppSettings["StartVirtualServerServiceOnAppLaunch"]) &&
                ConfigurationManager.AppSettings["StartVirtualServerServiceOnAppLaunch"].ToUpper() == "TRUE")
            {
                ServiceHelper.Start("Virtual Server");
            }

            if (ServiceHelper.IsRunning("Virtual Server"))
            {
                StartUserInterfaceThread();
            }
            else
            {
                MessageBox.Show("Virtual Server does not appear to be running. Please verify that the Virtual Server Service is started on the host specified in the configuration file.", "Virtual Server Service Status", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private static void StartUserInterfaceThread()
        {
            Thread uiThread = new Thread(new ThreadStart(InitUI));
            uiThread.Name = "UI Thread created from Main";
            uiThread.SetApartmentState(ApartmentState.STA);
            uiThread.IsBackground = false;
            uiThread.Start();
        }

		/// <summary>
		/// The equivalent to public static void Main, but without the STAThread
		/// attribute and unable to interfere with the COM Interop security - being
		/// initialized before this method is invoked.
		/// </summary>
		private static void InitUI()
		{
			Application.Run(new VirtualMachineConsoleForm());
		}

	}
}