using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Delsink.VirtualServer;
using Delsink;
using System.Configuration;

namespace Bliss.VPC
{
	[Author("Anton Delsink", "anton@delsink.com")]
	public partial class VirtualMachineActionsUserControl : UserControl
	{
		#region VirtualMachine
		private IVirtualMachine _VirtualMachine;
		public IVirtualMachine VirtualMachine
		{
			get
			{
				return _VirtualMachine;
			}
			set
			{
				_VirtualMachine = value;
				CreateActionButtons();
			}
		}
		#endregion

		#region StartVmrcOnVirtualMachineStart
		private bool defaultStartVmrcOnVirtualMachineStart = default(bool);
		private bool? _StartVmrcOnVirtualMachineStart = null;
		public bool StartVmrcOnVirtualMachineStart
		{
			get
			{
				if (_StartVmrcOnVirtualMachineStart == null)
				{
					bool parseResult;
					if (bool.TryParse(ConfigurationManager.AppSettings["StartVmrcOnVirtualMachineStart"], out parseResult))
					{
						_StartVmrcOnVirtualMachineStart = parseResult;
					}
				}
				return this._StartVmrcOnVirtualMachineStart.HasValue ? this._StartVmrcOnVirtualMachineStart.Value : this.defaultStartVmrcOnVirtualMachineStart;
			}
			set
			{
				_StartVmrcOnVirtualMachineStart = value;
			}
		}
		#endregion

		#region StartVmrcAfterVirtualMachineStartCompletes
		private bool defaultStartVmrcAfterVirtualMachineStartCompletes = default(bool);
		private bool? _StartVmrcAfterVirtualMachineStartCompletes = null;
		public bool StartVmrcAfterVirtualMachineStartCompletes
		{
			get
			{
				if (_StartVmrcAfterVirtualMachineStartCompletes == null)
				{
					bool parseResult;
					if (bool.TryParse(ConfigurationManager.AppSettings["StartVmrcAfterVirtualMachineStartCompletes"], out parseResult))
					{
						_StartVmrcAfterVirtualMachineStartCompletes = parseResult;
					}
				}
				return this._StartVmrcAfterVirtualMachineStartCompletes.HasValue ? this._StartVmrcAfterVirtualMachineStartCompletes.Value : this.defaultStartVmrcAfterVirtualMachineStartCompletes;
			}
			set
			{
				_StartVmrcAfterVirtualMachineStartCompletes = value;
			}
		}
		#endregion

		#region MaxWaitForVirtualMachineStartBeforeVmrcConnect
		private int defaultMaxWaitForVirtualMachineStartBeforeVmrcConnect = default(int);
		private int? _MaxWaitForVirtualMachineStartBeforeVmrcConnect = null;
		public int MaxWaitForVirtualMachineStartBeforeVmrcConnect
		{
			get
			{
				if (_MaxWaitForVirtualMachineStartBeforeVmrcConnect == null)
				{
					int parseResult;
					if (int.TryParse(ConfigurationManager.AppSettings["MaxWaitForVirtualMachineStartBeforeVmrcConnect"], out parseResult))
					{
						_MaxWaitForVirtualMachineStartBeforeVmrcConnect = parseResult;
					}
				}
				return this._MaxWaitForVirtualMachineStartBeforeVmrcConnect.HasValue ? this._MaxWaitForVirtualMachineStartBeforeVmrcConnect.Value : this.defaultMaxWaitForVirtualMachineStartBeforeVmrcConnect;
			}
			set
			{
				_MaxWaitForVirtualMachineStartBeforeVmrcConnect = value;
			}
		}
		#endregion
	
		public VirtualMachineActionsUserControl()
		{
			InitializeComponent();
		}

		public void UpdateUI()
		{
			CreateActionButtons();
		}
		private void CreateActionButtons()
		{
			flpButtons.SuspendLayout();
			flpButtons.Controls.Clear();
			
			if (this._VirtualMachine != null)
			{
				// Add "always available" buttons here:
				AddMountIsoButton();

				// Add "subject to status" buttons under the appropriate switch below:
				switch (this._VirtualMachine.State)
				{
					case VirtualMachineState.Paused:
						AddResumeButton();
						break;
					case VirtualMachineState.Saved:
						AddStartButton();
						if (this.VirtualMachine.State == VirtualMachineState.Saved) AddDiscardSavedStateButton();
						if (this.VirtualMachine.Undoable) AddDiscardUndoDisksButton();
						if (this.VirtualMachine.Undoable) AddMergeUndoDisksButton();
						
						break;
					case VirtualMachineState.TurnedOff:
						AddStartButton();
						if (this.VirtualMachine.Undoable) AddDiscardUndoDisksButton();
						if (this.VirtualMachine.Undoable) AddMergeUndoDisksButton();
						break;
					case VirtualMachineState.Running:
						AddSaveStateButton();
						AddTurnOffButton();
						AddPauseButton();
						break;
					case VirtualMachineState.TurningOn:
					case VirtualMachineState.TurningOff:
					case VirtualMachineState.Saving:
					case VirtualMachineState.Restoring:
					case VirtualMachineState.MergingDrives:
					case VirtualMachineState.Invalid:
					case VirtualMachineState.DeleteMachine:
						// nothing to do when in these states
						break;
				}
			}
			flpButtons.ResumeLayout();
		}
		
		#region Add ... Button
		private void AddStartButton()
		{
			Button btnStart = new Button();
			btnStart.Text = "&Start";
			btnStart.Click += new EventHandler(btnStart_Click);
			flpButtons.Controls.Add(btnStart);
		}
		private void AddSaveStateButton()
		{
			Button btnSaveState = new Button();
			btnSaveState.Text = "Save State";
			btnSaveState.Click += new EventHandler(btnSaveState_Click);
			flpButtons.Controls.Add(btnSaveState);
		}
		private void AddDiscardUndoDisksButton()
		{
			Button btnDiscardUndoDisks = new Button();
			btnDiscardUndoDisks.Text = "Discard";
			btnDiscardUndoDisks.Click += new EventHandler(btnDiscardUndoDisks_Click);
			flpButtons.Controls.Add(btnDiscardUndoDisks);
		}
		private void AddTurnOffButton()
		{
			Button btnTurnOff = new Button();
			btnTurnOff.Text = "Turn Off";
			btnTurnOff.Click += new EventHandler(btnTurnOff_Click);
			flpButtons.Controls.Add(btnTurnOff);
		}
		private void AddMergeUndoDisksButton()
		{
			Button btnMergeUndoDisks = new Button();
			btnMergeUndoDisks.Text = "Merge";
			btnMergeUndoDisks.Click += new EventHandler(btnMergeUndoDisks_Click);
			flpButtons.Controls.Add(btnMergeUndoDisks);
		}
		private void AddResumeButton()
		{
			Button btnResumeButton = new Button();
			btnResumeButton.Text = "Resume";
			btnResumeButton.Click += new EventHandler(btnResumeButton_Click);
			flpButtons.Controls.Add(btnResumeButton);
		}
		private void AddPauseButton()
		{
			Button btnPause = new Button();
			btnPause.Text = "Pause";
			btnPause.Click += new EventHandler(btnPause_Click);
			flpButtons.Controls.Add(btnPause);
		}
		private void AddDiscardSavedStateButton()
		{
			Button btnDiscardSavedState = new Button();
			btnDiscardSavedState.Text = "Discard Saved State";
			btnDiscardSavedState.Click += new EventHandler(btnDiscardSavedState_Click);
			flpButtons.Controls.Add(btnDiscardSavedState);
		}
		private void AddMountIsoButton()
		{
			Button btnMountISO = new Button();
			btnMountISO.Text = "Mount ISO";
			btnMountISO.Click += new EventHandler(btnMountISO_Click);
			flpButtons.Controls.Add(btnMountISO);
		}
		#endregion

		void btnResumeButton_Click(object sender, EventArgs e)
		{
			if (this._VirtualMachine != null)
			{
				this._VirtualMachine.Resume();
			}
		}
		void btnStart_Click(object sender, EventArgs e)
		{
			if (this._VirtualMachine != null)
			{
				ITask taskStarting = this._VirtualMachine.Start();

				if (StartVmrcOnVirtualMachineStart)
				{
					if (StartVmrcAfterVirtualMachineStartCompletes)
					{
						try
						{
							taskStarting.WaitForCompletion(MaxWaitForVirtualMachineStartBeforeVmrcConnect);
						}
						catch
						{
							MessageBox.Show("Virtual Machine did not complete startup within the configured time limit. Please connect VMRC manually.");
						}
					}

					VirtualServerManager.StartVmrc("vmrc://" + Environment.MachineName + "/" + this._VirtualMachine.Name);
				}
			}
		}
		void btnPause_Click(object sender, EventArgs e)
		{
			if (this._VirtualMachine != null)
			{
				this._VirtualMachine.Pause();
			}
		}
		void btnResume_Click(object sender, EventArgs e)
		{
			if (this._VirtualMachine != null)
			{
				this._VirtualMachine.Resume();
			}
		}
		void btnReset_Click(object sender, EventArgs e)
		{
			if (this._VirtualMachine != null)
			{
				this._VirtualMachine.Reset();
			}
		}
		void btnMergeUndoDisks_Click(object sender, EventArgs e)
		{
			if (this._VirtualMachine != null)
			{
				this._VirtualMachine.MergeUndoDisks();
			}
		}
		void btnDiscardUndoDisks_Click(object sender, EventArgs e)
		{
			if (this._VirtualMachine != null)
			{
				this._VirtualMachine.DiscardUndoDisks();
			}
		}
		void btnDiscardSavedState_Click(object sender, EventArgs e)
		{
			if (this._VirtualMachine != null)
			{
				this._VirtualMachine.DiscardSavedState();
			}
		}
		void btnSaveState_Click(object sender, EventArgs e)
		{
			if (this._VirtualMachine != null)
			{
				this._VirtualMachine.SaveState();
			}
		}
		void btnTurnOff_Click(object sender, EventArgs e)
		{
			if (this._VirtualMachine != null)
			{
				this._VirtualMachine.TurnOff();
			}
		}
		void btnMountISO_Click(object sender, EventArgs e)
		{
			if (this.VirtualMachine != null)
			{
				if (dlgOpenFile.ShowDialog() == DialogResult.OK)
				{
					if (this.VirtualMachine.DvdDrives.Count > 0)
					{
						this.VirtualMachine.DvdDrives[0].AttachISO(dlgOpenFile.FileName);
					}
				}
			}
		}
	}
}
