using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Delsink.VirtualServer;
using System.Configuration;

namespace Bliss.VPC
{
	/// <summary>
	/// For demonstration purposes only; to be replaced by a control that closely mimicks the look of VPC
	/// </summary>
	internal partial class VirtualMachineUserControl : UserControl
	{
		public VirtualMachineUserControl()
		{
			InitializeComponent();
		}

		#region VM
		private IVirtualMachine _VM;
		public IVirtualMachine VM
		{
			get
			{
				return _VM;
			}
			set
			{
				_VM = value;

				UpdateUI();
			}
		}
		#endregion

		#region IsSelected
		private bool _IsSelected;
		public bool IsSelected
		{
			get
			{
				return _IsSelected;
			}
			set
			{
				_IsSelected = value;
				if (_IsSelected)
				{
					this.BackColor = SystemColors.Highlight;
				}
				else
				{
					this.BackColor = SystemColors.Control;
				}
			}
		}
		#endregion


		#region DisplayThumbnail
		private bool defaultDisplayThumbnail = default(bool);
		private bool? _DisplayThumbnail = null;
		public bool DisplayThumbnail
		{
			get
			{
				if (_DisplayThumbnail == null)
				{
					bool parseResult;
					if (bool.TryParse(ConfigurationManager.AppSettings["DisplayVirtualMachineThumbnail"], out parseResult))
					{
						_DisplayThumbnail = parseResult;
					}
				}
				return this._DisplayThumbnail.HasValue ? this._DisplayThumbnail.Value : this.defaultDisplayThumbnail;
			}
			set
			{
				_DisplayThumbnail = value;
			}
		}
		#endregion

		private void VirtualMachineUserControl_Load(object sender, EventArgs e)
		{
		}
		public void UpdateUI()
		{
			this.UpdateUI(this.DisplayThumbnail);
		}
		public void UpdateUI(bool displayThumbnail)
		{
			this.lblVirtualMachineName.Text = this.VM.Name;
			this.lblVirtualMachineStatus.Text = this.VM.State.ToString();

			if (displayThumbnail)
			{
				this.bgwThumbnail.RunWorkerAsync();
			}
		}

		private void VirtualMachineUserControl_Click(object sender, EventArgs e)
		{
			OnSelect();
		}
		public void OnSelect()
		{
			this.IsSelected = !this.IsSelected;

			Selected(this, new EventArgs());
		}
		public delegate void VirtualMachineUserControlSelectedDelegate(object sender, EventArgs e);
		public event VirtualMachineUserControlSelectedDelegate Selected;

		private void bgwThumbnail_DoWork(object sender, DoWorkEventArgs e)
		{
			e.Result = this.VM.Thumbnail;
		}

		private void bgwThumbnail_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			this.picThumbnail.BackgroundImage = e.Result as Bitmap;
		}
	}
}
